"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("@src/api");
// The history methods 'pushState' and 'replaceState' by default do not fire an event
// unless it is coming from user interaction with the browser navigation bar,
// so we are adding a patch to make them detectable
var isPatched = false;
var patchHistoryMethodsOnce = function () {
    if (isPatched)
        return;
    var methods = ['pushState', 'replaceState'];
    methods.forEach(function (method) {
        var original = history[method];
        history[method] = function (state) {
            // eslint-disable-next-line prefer-rest-params
            var result = original.apply(this, arguments);
            var event = new Event(method.toLowerCase());
            event.state = state;
            window.dispatchEvent(event);
            return result;
        };
    });
    isPatched = true;
};
function useLocation(runOnMount) {
    if (runOnMount === void 0) { runOnMount = true; }
    var buildState = function (trigger) {
        var state = history.state, length = history.length;
        var hash = location.hash, host = location.host, hostname = location.hostname, href = location.href, origin = location.origin, pathname = location.pathname, port = location.port, protocol = location.protocol, search = location.search;
        return {
            trigger: trigger,
            state: state,
            length: length,
            hash: hash,
            host: host,
            hostname: hostname,
            href: href,
            origin: origin,
            pathname: pathname,
            port: port,
            protocol: protocol,
            search: search
        };
    };
    var isTracking = api_1.ref(false);
    var locationState = api_1.ref(buildState('load'));
    var popState = function () { return (locationState.value = buildState('popstate')); };
    var pushState = function () { return (locationState.value = buildState('pushstate')); };
    var replaceState = function () { return (locationState.value = buildState('replacestate')); };
    var start = function () {
        patchHistoryMethodsOnce();
        if (isTracking.value)
            return;
        isTracking.value = true;
        locationState.value = buildState('start');
        window.addEventListener('popstate', popState);
        window.addEventListener('pushstate', pushState);
        window.addEventListener('replacestate', replaceState);
    };
    var stop = function () {
        if (!isTracking.value)
            return;
        isTracking.value = false;
        window.removeEventListener('popstate', popState);
        window.removeEventListener('pushstate', pushState);
        window.removeEventListener('replacestate', replaceState);
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return { locationState: locationState, isTracking: isTracking, start: start, stop: stop };
}
exports.useLocation = useLocation;
